/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was com.canonical.indicators.webcredentials.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#ifndef WEBCREDENTIALS_ADAPTOR_H
#define WEBCREDENTIALS_ADAPTOR_H

#include <QtCore/QObject>
#include <QtDBus/QtDBus>
#include <QtCore/qcontainerfwd.h>

/*
 * Adaptor class for interface com.canonical.indicators.webcredentials
 */
class WebcredentialsAdaptor: public QDBusAbstractAdaptor
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "com.canonical.indicators.webcredentials")
    Q_CLASSINFO("D-Bus Introspection", ""
"  <interface name=\"com.canonical.indicators.webcredentials\">\n"
"    <method name=\"ReportFailure\">\n"
"      <annotation value=\"QVariantMap\" name=\"org.qtproject.QtDBus.QtTypeName.In1\"/>\n"
"      <arg direction=\"in\" type=\"u\" name=\"account_id\"/>\n"
"      <arg direction=\"in\" type=\"a{sv}\" name=\"notification\"/>\n"
"    </method>\n"
"    <method name=\"RemoveFailures\">\n"
"      <annotation value=\"QSet&lt;uint&gt;\" name=\"org.qtproject.QtDBus.QtTypeName.In0\"/>\n"
"      <arg direction=\"in\" type=\"au\" name=\"account_ids\"/>\n"
"    </method>\n"
"    <method name=\"ReauthenticateAccount\">\n"
"      <annotation value=\"QVariantMap\" name=\"org.qtproject.QtDBus.QtTypeName.In1\"/>\n"
"      <arg direction=\"in\" type=\"u\" name=\"account_id\"/>\n"
"      <arg direction=\"in\" type=\"a{sv}\" name=\"extra_parameters\"/>\n"
"      <arg direction=\"out\" type=\"b\" name=\"reauthenticated\"/>\n"
"    </method>\n"
"    <method name=\"ClearErrorStatus\"/>\n"
"    <property access=\"read\" type=\"au\" name=\"Failures\">\n"
"      <annotation value=\"QSet&lt;uint&gt;\" name=\"org.qtproject.QtDBus.QtTypeName\"/>\n"
"    </property>\n"
"    <property access=\"read\" type=\"b\" name=\"ErrorStatus\"/>\n"
"  </interface>\n"
        "")
public:
    WebcredentialsAdaptor(QObject *parent);
    ~WebcredentialsAdaptor() override;

public: // PROPERTIES
    Q_PROPERTY(bool ErrorStatus READ errorStatus)
    bool errorStatus() const;

    Q_PROPERTY(QSet<uint> Failures READ failures)
    QSet<uint> failures() const;

public Q_SLOTS: // METHODS
    void ClearErrorStatus();
    bool ReauthenticateAccount(uint account_id, const QVariantMap &extra_parameters);
    void RemoveFailures(const QSet<uint> &account_ids);
    void ReportFailure(uint account_id, const QVariantMap &notification);
Q_SIGNALS: // SIGNALS
};

#endif
