/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was com.canonical.indicators.webcredentials.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "indicator-service.h"
#include "webcredentials_adaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class WebcredentialsAdaptor
 */

WebcredentialsAdaptor::WebcredentialsAdaptor(QObject *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

WebcredentialsAdaptor::~WebcredentialsAdaptor()
{
    // destructor
}

bool WebcredentialsAdaptor::errorStatus() const
{
    // get the value of property ErrorStatus
    return qvariant_cast< bool >(parent()->property("ErrorStatus"));
}

QSet<uint> WebcredentialsAdaptor::failures() const
{
    // get the value of property Failures
    return qvariant_cast< QSet<uint> >(parent()->property("Failures"));
}

void WebcredentialsAdaptor::ClearErrorStatus()
{
    // handle method call com.canonical.indicators.webcredentials.ClearErrorStatus
    QMetaObject::invokeMethod(parent(), "ClearErrorStatus");
}

bool WebcredentialsAdaptor::ReauthenticateAccount(uint account_id, const QVariantMap &extra_parameters)
{
    // handle method call com.canonical.indicators.webcredentials.ReauthenticateAccount
    bool reauthenticated{};
    QMetaObject::invokeMethod(parent(), "ReauthenticateAccount", Q_RETURN_ARG(bool, reauthenticated), Q_ARG(uint, account_id), Q_ARG(QVariantMap, extra_parameters));
    return reauthenticated;
}

void WebcredentialsAdaptor::RemoveFailures(const QSet<uint> &account_ids)
{
    // handle method call com.canonical.indicators.webcredentials.RemoveFailures
    QMetaObject::invokeMethod(parent(), "RemoveFailures", Q_ARG(QSet<uint>, account_ids));
}

void WebcredentialsAdaptor::ReportFailure(uint account_id, const QVariantMap &notification)
{
    // handle method call com.canonical.indicators.webcredentials.ReportFailure
    QMetaObject::invokeMethod(parent(), "ReportFailure", Q_ARG(uint, account_id), Q_ARG(QVariantMap, notification));
}

