/****************************************************************************
** Meta object code from reading C++ file 'webcredentials_adaptor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "webcredentials_adaptor.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QSet>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'webcredentials_adaptor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN21WebcredentialsAdaptorE_t {};
} // unnamed namespace

template <> constexpr inline auto WebcredentialsAdaptor::qt_create_metaobjectdata<qt_meta_tag_ZN21WebcredentialsAdaptorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "WebcredentialsAdaptor",
        "D-Bus Interface",
        "com.canonical.indicators.webcredentials",
        "D-Bus Introspection",
        "  <interface name=\"com.canonical.indicators.webcredentials\">\n  "
        "  <method name=\"ReportFailure\">\n      <annotation value=\"QVari"
        "antMap\" name=\"org.qtproject.QtDBus.QtTypeName.In1\"/>\n      <ar"
        "g direction=\"in\" type=\"u\" name=\"account_id\"/>\n      <arg di"
        "rection=\"in\" type=\"a{sv}\" name=\"notification\"/>\n    </metho"
        "d>\n    <method name=\"RemoveFailures\">\n      <annotation value="
        "\"QSet&lt;uint&gt;\" name=\"org.qtproject.QtDBus.QtTypeName.In0\"/"
        ">\n      <arg direction=\"in\" type=\"au\" name=\"account_ids\"/>\n"
        "    </method>\n    <method name=\"ReauthenticateAccount\">\n      "
        "<annotation value=\"QVariantMap\" name=\"org.qtproject.QtDBus.QtTy"
        "peName.In1\"/>\n      <arg direction=\"in\" type=\"u\" name=\"acco"
        "unt_id\"/>\n      <arg direction=\"in\" type=\"a{sv}\" name=\"extr"
        "a_parameters\"/>\n      <arg direction=\"out\" type=\"b\" name=\"r"
        "eauthenticated\"/>\n    </method>\n    <method name=\"ClearErrorSt"
        "atus\"/>\n    <property access=\"read\" type=\"au\" name=\"Failure"
        "s\">\n      <annotation value=\"QSet&lt;uint&gt;\" name=\"org.qtpr"
        "oject.QtDBus.QtTypeName\"/>\n    </property>\n    <property access"
        "=\"read\" type=\"b\" name=\"ErrorStatus\"/>\n  </interface>\n",
        "ClearErrorStatus",
        "",
        "ReauthenticateAccount",
        "account_id",
        "QVariantMap",
        "extra_parameters",
        "RemoveFailures",
        "QSet<uint>",
        "account_ids",
        "ReportFailure",
        "notification",
        "ErrorStatus",
        "Failures"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'ClearErrorStatus'
        QtMocHelpers::SlotData<void()>(5, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'ReauthenticateAccount'
        QtMocHelpers::SlotData<bool(uint, const QVariantMap &)>(7, 6, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::UInt, 8 }, { 0x80000000 | 9, 10 },
        }}),
        // Slot 'RemoveFailures'
        QtMocHelpers::SlotData<void(const QSet<uint> &)>(11, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Slot 'ReportFailure'
        QtMocHelpers::SlotData<void(uint, const QVariantMap &)>(14, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 8 }, { 0x80000000 | 9, 15 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'ErrorStatus'
        QtMocHelpers::PropertyData<bool>(16, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'Failures'
        QtMocHelpers::PropertyData<QSet<uint>>(17, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<WebcredentialsAdaptor, qt_meta_tag_ZN21WebcredentialsAdaptorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject WebcredentialsAdaptor::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractAdaptor::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21WebcredentialsAdaptorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21WebcredentialsAdaptorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN21WebcredentialsAdaptorE_t>.metaTypes,
    nullptr
} };

void WebcredentialsAdaptor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WebcredentialsAdaptor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->ClearErrorStatus(); break;
        case 1: { bool _r = _t->ReauthenticateAccount((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 2: _t->RemoveFailures((*reinterpret_cast<std::add_pointer_t<QSet<uint>>>(_a[1]))); break;
        case 3: _t->ReportFailure((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSet<uint> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QSet<uint> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->errorStatus(); break;
        case 1: *reinterpret_cast<QSet<uint>*>(_v) = _t->failures(); break;
        default: break;
        }
    }
}

const QMetaObject *WebcredentialsAdaptor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *WebcredentialsAdaptor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21WebcredentialsAdaptorE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractAdaptor::qt_metacast(_clname);
}

int WebcredentialsAdaptor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractAdaptor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}
QT_WARNING_POP
