/****************************************************************************
** Resource object code
**
** Created by: The Resource Compiler for Qt version 6.11.1
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#ifdef _MSC_VER
// disable informational message "function ... selected for automatic inline expansion"
#pragma warning (disable: 4711)
#endif

static const unsigned char qt_resource_data[] = {
  // security-low.png
  0x0,0x0,0x1,0xeb,
  0x89,
  0x50,0x4e,0x47,0xd,0xa,0x1a,0xa,0x0,0x0,0x0,0xd,0x49,0x48,0x44,0x52,0x0,
  0x0,0x0,0x10,0x0,0x0,0x0,0x10,0x8,0x6,0x0,0x0,0x0,0x1f,0xf3,0xff,0x61,
  0x0,0x0,0x0,0x4,0x73,0x42,0x49,0x54,0x8,0x8,0x8,0x8,0x7c,0x8,0x64,0x88,
  0x0,0x0,0x0,0x9,0x70,0x48,0x59,0x73,0x0,0x0,0xd,0xd7,0x0,0x0,0xd,0xd7,
  0x1,0x42,0x28,0x9b,0x78,0x0,0x0,0x0,0x19,0x74,0x45,0x58,0x74,0x53,0x6f,0x66,
  0x74,0x77,0x61,0x72,0x65,0x0,0x77,0x77,0x77,0x2e,0x69,0x6e,0x6b,0x73,0x63,0x61,
  0x70,0x65,0x2e,0x6f,0x72,0x67,0x9b,0xee,0x3c,0x1a,0x0,0x0,0x1,0x68,0x49,0x44,
  0x41,0x54,0x38,0x8d,0x75,0x90,0xb1,0x4a,0x3,0x41,0x10,0x86,0xbf,0xbd,0x78,0xe4,
  0x50,0x50,0x53,0xd8,0xa4,0x13,0x89,0xa6,0xd0,0x67,0x10,0x14,0x7c,0xb,0x4b,0x11,
  0x4c,0xad,0x39,0xb8,0x66,0x2e,0x55,0x62,0x52,0x5,0x22,0x18,0xb0,0xb1,0xb7,0x50,
  0x7b,0xf1,0x1,0x7c,0x0,0x3b,0x1b,0x21,0x85,0x48,0x40,0x90,0x53,0xe1,0x76,0x2d,
  0xdc,0x84,0xcd,0x99,0x1d,0x58,0x96,0x9d,0x99,0xff,0xdb,0x7f,0x46,0x1,0xb4,0x5a,
  0xad,0x1d,0xad,0x75,0xf,0xa8,0x1,0xab,0x40,0x5,0x7f,0x7c,0x0,0x4f,0x40,0x43,
  0x44,0x9e,0x3,0x0,0xad,0xf5,0x25,0x70,0x0,0xac,0x5b,0x71,0xe,0x9c,0x3,0x5b,
  0xa5,0x52,0xa9,0xe,0xf4,0x0,0x6d,0x1,0xcb,0xc0,0x1e,0x30,0x0,0x58,0xb0,0xc9,
  0x5a,0xe1,0x97,0x5e,0x10,0x4,0xb7,0x5a,0xeb,0x9b,0x3c,0xcf,0x1,0x8e,0x1,0x3,
  0x9c,0x3a,0x3d,0x9b,0x0,0x81,0x7d,0xac,0x14,0x0,0x43,0xad,0xf5,0x15,0xb0,0x6d,
  0xcf,0x35,0x30,0x2c,0xf4,0xac,0xb9,0x80,0xd0,0xad,0x88,0xc8,0xb,0x50,0x77,0x52,
  0x1b,0xc0,0x6b,0x1,0x50,0x76,0x1,0xc5,0x30,0x80,0xf2,0xd4,0x66,0x62,0x6,0xa0,
  0x94,0x3a,0xb,0xc3,0xb0,0x6a,0x5d,0x28,0xa5,0x54,0x6c,0x4b,0x1d,0x11,0xf9,0xa,
  0xc3,0xb0,0xea,0xe4,0xfe,0x3,0x8c,0x31,0x83,0x24,0x49,0x46,0x0,0x69,0x9a,0x36,
  0x8d,0x31,0x6d,0x2b,0x8e,0x1,0x92,0x24,0x19,0x45,0x51,0x74,0xe1,0x5,0x88,0x48,
  0xe6,0xc0,0x2a,0xd6,0xd5,0xd8,0xed,0xc9,0xb2,0xec,0xd3,0xb,0xe0,0x6f,0xf6,0x9,
  0x2c,0x6,0x52,0x63,0x4c,0x5b,0x44,0xc4,0xc9,0x6b,0x57,0xe0,0x5b,0x22,0xf6,0xf7,
  0x89,0xa3,0xc8,0xd7,0x53,0x4,0x4c,0x37,0x3f,0x6f,0x7,0xd6,0xc1,0x8c,0x66,0xf2,
  0xf8,0xb1,0xc5,0xf2,0x74,0x16,0xcf,0xe,0x80,0x45,0x7b,0x7f,0xbb,0x80,0x77,0xdb,
  0x7c,0xd8,0xed,0x76,0x97,0x2c,0x2c,0x6,0x3a,0xc6,0x98,0x76,0x9a,0xa6,0x4d,0x80,
  0x7e,0xbf,0x5f,0x56,0x4a,0x1d,0x59,0xcd,0xdb,0xd4,0xb2,0x88,0x3c,0x2,0xbb,0xbe,
  0x39,0x3d,0xf1,0x20,0x22,0xfb,0x13,0x7,0x27,0xc0,0x3d,0x50,0xb4,0x3b,0x2f,0xc6,
  0xc0,0x1d,0xd0,0x0,0xf8,0x5,0xa6,0x40,0x7d,0x2d,0x9b,0x90,0x44,0xe9,0x0,0x0,
  0x0,0x0,0x49,0x45,0x4e,0x44,0xae,0x42,0x60,0x82,
  
};

static const unsigned char qt_resource_name[] = {
  // security-low.png
  0x0,0x10,
  0xa,0x59,0x48,0x67,
  0x0,0x73,
  0x0,0x65,0x0,0x63,0x0,0x75,0x0,0x72,0x0,0x69,0x0,0x74,0x0,0x79,0x0,0x2d,0x0,0x6c,0x0,0x6f,0x0,0x77,0x0,0x2e,0x0,0x70,0x0,0x6e,0x0,0x67,
  
};

static const unsigned char qt_resource_struct[] = {
  // :
  0x0,0x0,0x0,0x0,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x1,
0x0,0x0,0x1,0x9e,0x53,0xca,0xc4,0x50,
  // :/security-low.png
  0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x0,
0x0,0x0,0x1,0x9e,0x53,0xca,0xc4,0x50,

};

#ifdef QT_NAMESPACE
#  define QT_RCC_PREPEND_NAMESPACE(name) ::QT_NAMESPACE::name
#  define QT_RCC_MANGLE_NAMESPACE0(x) x
#  define QT_RCC_MANGLE_NAMESPACE1(a, b) a##_##b
#  define QT_RCC_MANGLE_NAMESPACE2(a, b) QT_RCC_MANGLE_NAMESPACE1(a,b)
#  define QT_RCC_MANGLE_NAMESPACE(name) QT_RCC_MANGLE_NAMESPACE2( \
        QT_RCC_MANGLE_NAMESPACE0(name), QT_RCC_MANGLE_NAMESPACE0(QT_NAMESPACE))
#else
#   define QT_RCC_PREPEND_NAMESPACE(name) name
#   define QT_RCC_MANGLE_NAMESPACE(name) name
#endif

#if defined(QT_INLINE_NAMESPACE)
inline namespace QT_NAMESPACE {
#elif defined(QT_NAMESPACE)
namespace QT_NAMESPACE {
#endif

bool qRegisterResourceData(int, const unsigned char *, const unsigned char *, const unsigned char *);
bool qUnregisterResourceData(int, const unsigned char *, const unsigned char *, const unsigned char *);

#ifdef QT_NAMESPACE
}
#endif

int QT_RCC_MANGLE_NAMESPACE(qInitResources_http_warning)();
int QT_RCC_MANGLE_NAMESPACE(qInitResources_http_warning)()
{
    int version = 3;
    QT_RCC_PREPEND_NAMESPACE(qRegisterResourceData)
        (version, qt_resource_struct, qt_resource_name, qt_resource_data);
    return 1;
}

int QT_RCC_MANGLE_NAMESPACE(qCleanupResources_http_warning)();
int QT_RCC_MANGLE_NAMESPACE(qCleanupResources_http_warning)()
{
    int version = 3;
    QT_RCC_PREPEND_NAMESPACE(qUnregisterResourceData)
       (version, qt_resource_struct, qt_resource_name, qt_resource_data);
    return 1;
}

#ifdef __clang__
#   pragma clang diagnostic push
#   pragma clang diagnostic ignored "-Wexit-time-destructors"
#endif

namespace {
   struct initializer {
       initializer() { QT_RCC_MANGLE_NAMESPACE(qInitResources_http_warning)(); }
       ~initializer() { QT_RCC_MANGLE_NAMESPACE(qCleanupResources_http_warning)(); }
   } dummy;
}

#ifdef __clang__
#   pragma clang diagnostic pop
#endif
